@@ -2,6 +2,7 @@
    
    Copyright (C) 1995 Werner Almesberger
    		 1996,1998 Jakub Jelinek
+		 2001 Ben Collins
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,14 +16,21 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   USA.  */
 
-#include "silo.h"
-#include "setjmp.h"
+/* This file gets included by the silo program (userspace), try to be nice */
+#ifndef SILOCONFCHECK
+
+#include <silo.h>
+#include <stringops.h>
+#include <setjmp.h>
 #ifndef NULL
 #define NULL (void *)0
 #endif
-#include "promlib.h"
+#include <promlib.h>
+
+#endif /* !SILOCONFCHECK */
 
 typedef enum {
     cft_strg, cft_flag, cft_end
@@ -36,7 +44,9 @@ typedef struct {
 
 #define MAX_TOKEN 200
 #define MAX_VAR_NAME MAX_TOKEN
+#ifndef EOF
 #define EOF -1
+#endif
 
 CONFIG cf_options[] =
 {
@@ -114,7 +124,7 @@ void cfg_error (char *msg,...)
     longjmp (env, 1);
 }
 
-inline int getc ()
+inline int my_getc ()
 {
     if (currp == endp)
 	return EOF;
@@ -127,7 +137,7 @@ static int next (void)
     int ch;
 
     if (!back)
-	return getc ();
+	return my_getc ();
     ch = back;
     back = 0;
     return ch;
@@ -334,7 +344,6 @@ int cfg_parse (char *cfg_file, char *buff, int len)
     file_name = cfg_file;
     currp = buff;
     endp = currp + len;
-
     if (setjmp (env))
 	return -1;
     while (1) {
@@ -389,26 +398,37 @@ int cfg_get_flag (char *image, char *item)
 }
 
 static int printl_count = 0;
-static void printlabel (char *label)
+static void printlabel (const char *label, char **addr, const char *match)
 {
     int len = strlen (label);
 
-    if (!printl_count)
-	printf ("\n");
-    printf ("%s", label);
-    while (len++ < 25)
-	putchar (' ');
-    printl_count++;
-    if (printl_count == 3)
-	printl_count = 0;
+    if (match == NULL ||
+	(strlen(label) >= strlen(match) &&
+	!strncmp(match, label, strlen(match)))) {
+
+	if (*addr != NULL) {
+	    strcpy(*addr, label);
+	    *addr += strlen(label) + 1;
+	    **addr = '\0';
+	} else {
+	    if (!(printl_count % 3))
+		printf ("\n");
+	    printf ("%s", label);
+	    while (len++ < 25)
+		putchar (' ');
+	}
+	printl_count++;
+    }
 }
 
-void cfg_print_images (void)
+int cfg_print_images (char *addr, char *match)
 {
     struct IMAGES *p;
     char *label, *alias;
 
     printl_count = 0;
+    if (addr) addr[0] = '\0';
+ 
     for (p = images; p; p = p->next) {
 	label = cfg_get_strg_i (p->table, "label");
 	if (!label) {
@@ -418,11 +438,13 @@ void cfg_print_images (void)
 		label = alias + 1;
 	}
 	alias = cfg_get_strg_i (p->table, "alias");
-	printlabel (label);
+	printlabel(label, &addr, match);
 	if (alias)
-	    printlabel (alias);
+	    printlabel (alias, &addr, match);
     }
-    printf ("\n");
+    if (!addr && printl_count)
+	printf ("\n");
+    return printl_count;
 }
 
 char *cfg_get_default (void)
