@@ -15,10 +15,11 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   USA.  */
 
-#include "silo.h"
-#include "setjmp.h"
+#include <silo.h>
+#include <setjmp.h>
 #ifndef NULL
 #define NULL (void *)0
 #endif
@@ -75,7 +76,7 @@ jmp_buf gunzip_env;
 #define get_byte() (*get_input_fun)()
 #define unget_byte() (*unget_input_fun)()
 
-#include "inflate.c"
+#include "../common/inflate.c"
 
 static void error (char *m)
 {
@@ -136,14 +137,14 @@ static void unget_input(void)
     gzminp--;
 }
 
-extern char start, main_text_start, main_text_end, main_data_start, main_data_end, main_rodata_start, main_rodata_end, __bss_start;
+extern char main_text_start, main_text_end, main_data_start, main_data_end, main_rodata_start, main_rodata_end, __bss_start;
 
 /* This has to be in data section, so that it does not get cleared. See crt0.S for details. */
 char *gzminpi = (char *)0xdeadbeef;
 
 extern int bootmain(void);
 
-unsigned gunzipmain(struct linux_romvec *promvec, void *cifh, void *cifs)
+unsigned my_main(struct linux_romvec *promvec, void *cifh, void *cifs)
 {
     prom_init(promvec, cifh, cifs);
 
@@ -162,7 +163,7 @@ unsigned gunzipmain(struct linux_romvec *promvec, void *cifh, void *cifs)
     	gzminpi = (char *)pt;
     }
     gzminp = gzminpi;
-    if (decompress ((char *)0x200000, (char *)&start, get_input, unget_input) == -1) {
+    if (decompress ((char *)0x200000, (char *)&_start, get_input, unget_input) == -1) {
         printf ("\nInternal error\n");
         prom_halt();
     }
