@@ -15,16 +15,22 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   USA.  */
 
-#define VERSION "0.9.8"
-#define IMGVERSION "0.98"
+#if !defined(VERSION) || !defined(IMGVERSION)
+#error VERSION and IMGVERSION must be defined
+#endif
 #define DFL_CONFIG "/etc/silo.conf"
 #define DFL_BACKUP "/boot/old.b"
 #define DFL_PRIMARY "/boot/first.b"
 #define DFL_PRIMARY_U "/boot/ultra.b"
 #define DFL_SECONDARY "/boot/second.b"
 
+#ifdef __sun__
+#include "../second/fs/ufs.c"
+#endif
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -39,13 +45,12 @@
 #  include <sys/time.h>
 #  define _LINUX_STAT_H
 #  define _LINUX_VFS_H
+#  define _LINUX_TIME_H
+#  define _LINUX_STRING_H_
 #endif
 #ifdef __linux__
 #  include <linux/fs.h>
 #  include <linux/ext2_fs.h>
-#  ifdef __GLIBC__
-#    define _LINUX_TIME_H
-#  endif
 #  include <scsi/scsi.h>
 #  include <sys/vfs.h>
 #  ifdef __GLIBC__
@@ -59,7 +64,7 @@
 #    define mmakedev(major, minor) (((major) << 8) | (minor))
 #  endif
 #  include <md-int.h>
-#elif defined (__solaris__)
+#elif defined (__sun__)
 #  include <sys/types.h>
 #  include <sys/stat.h>
 #  include <non-linux/ext2_fs.h>
@@ -91,7 +96,10 @@ static int ufs_blocks (struct hwdevice *, ino_t);
 #define ULTRA_NUMBER_OFFSET (ULTRA_NUMBER_OFFSET_TMP + 0x223)
 #define FD_DIGIT_OFFSET (FD_DIGIT_OFFSET_TMP + 0x223)
 #define FD_LETTER_OFFSET (FD_LETTER_OFFSET_TMP + 0x223)
-#define IEEE32_OFFSET (IEEE32_OFFSET_TMP + 0x10)
+
+/* Checks our config file for errors */
+int confcheck(char*);
+static int allow_confchk_fail = 0;
 
 /* This is just so that we don't have to fight with incompatible ufs_fs.h headers */
 #define SILO_UFS_MAGIC 0x00011954
@@ -360,7 +368,7 @@ again:
     	if (p) p++; else p = name;
     	start = (long)st.st_ino + 16 + ((strlen(p) + 16) & ~15);
     	if (start & 511)
-    		fatal ("File %s on romfs not aligned on 512B boundary. Use genromfs -a -r to generate the image", name);
+    		fatal ("File %s on romfs not aligned on 512B boundary. Use genromfs -a to generate the image", name);
     	start = hwdev->doff + start / 512;
 	if (flash_image) start += 1024 / 512;	/* make room for ieee32 */
     	for (j = 0; j * 512 < size; j++)
@@ -375,7 +383,7 @@ again:
 	    break;
 	if (!block) {
 	    if ((j << 9) < size)
-	        fatal ("Filesystem holes are not yet supported for second stage loader. Mail jj@sunsite.mff.cuni.cz");
+	        fatal ("Filesystem holes are not yet supported for second stage loader. Mail silo-general@lists.sourceforge.net");
 	    else
 	        break;
 	}
@@ -436,7 +444,7 @@ again:
     	}
     	gpb_cleanup(filename, movecount - 1);
     }
-#elif defined(__solaris__)
+#elif defined(__sun__)
     ufs_blocks (hwdev, st.st_ino);
 #endif
     close (fd);
@@ -625,6 +633,7 @@ void usage (char *s)
             " -F             make a romfs bootable floppy\n"
             " -J image       make a romfs bootable flash image\n"
             "                -J and -F are incompatible\n"
+	    " -a             Allow silo.conf syntax check to fail\n"
             ,s);
     exit (1);
 }
@@ -726,7 +735,7 @@ void install_first_stage (char *device, char *filename)
         *(unsigned int *)(ush + 62) = (-d/2);
 
         if (write (fd, &sdl, 512) != 512)
-            fatal ("Couldn't write to %s your new partition table", device);
+            fatal ("Couldn't write new partition table to %s", device);
     } else if (flash_image) {
 	/*
 	 * Make sure that both block table address and checksum fit.
@@ -744,7 +753,7 @@ void install_first_stage (char *device, char *filename)
     } else if (lseek (fd, 512, 0) != 512)
         fatal ("Couldn't seek on %s", device);
     if (write (fd, buff, rc) != rc)
-        fatal ("Couldn't write to %s your new silo bootblock", device);
+        fatal ("Couldn't write new silo bootblock to %s", device);
     close (fd);
     fclose (fp);
 }
@@ -784,7 +793,7 @@ int get_prom_ver(int use_prom)
     	}
     }
     return ver;
-#elif defined(__solaris__)
+#elif defined(__sun__)
     int ver = -1;
     
     if (use_prom)
@@ -1017,7 +1026,7 @@ struct hwdevice *get_device(int majno, int minno)
 		if (!p) fatal ("Couldn't find out what device is second stage on");
 		strcpy (dev, p);
 		strcpy (wholedev, p);
-#ifdef __solaris__
+#ifdef __sun__
 		if (strlen (p) == strlen ("/dev/dsk/c0t0d0s0")) {
 		    p = strchr (p, 0) - 2;
 		    if (*p == 's' && p[1] >= '0' && p[1] <= '7') {
@@ -1058,7 +1067,7 @@ int main(int argc,char **argv)
     int use_prom = 0;
     int print_prom_version = 0;
 
-#ifdef __solaris__
+#ifdef __sun__
     if (prom_init () >= 0)
         use_prom = 1;
 #endif
@@ -1148,6 +1157,9 @@ int main(int argc,char **argv)
 		flash_image = *argv++;
 		argc--;
 		break;
+	    case 'a':
+		allow_confchk_fail = 1;
+		break;
 	    default:
 		usage(name);
 	}
@@ -1172,6 +1184,7 @@ int main(int argc,char **argv)
     }
     if (argc) usage(name);
     if (floppy_image && flash_image) usage(name);
+
     if (flash_image) {
 	char *p, *q;
 	if (*flash_image == '/') {
@@ -1202,6 +1215,10 @@ int main(int argc,char **argv)
        chdir("/");
     }
 
+    /* This _must_ be done after chrooting */
+    if (!confcheck(config_file) && !allow_confchk_fail)
+	return 1;
+
     secondary = strdup (secondary);
     if (stat (secondary, &st1) < 0)
         fatal ("Cannot open second stage loader %s", secondary);
@@ -1248,7 +1265,7 @@ int main(int argc,char **argv)
 #ifdef __linux__
         else if ((hwdevs->type == TYPE_SCSI && (mmajor(st2.st_dev) != mmajor(st1.st_dev) || (mminor(st2.st_dev) & (~0xf)) != (mminor(st1.st_dev) & (~0xf)))) ||
             (hwdevs->type == TYPE_IDE && (mmajor(st2.st_dev) != mmajor(st1.st_dev) || (mminor(st2.st_dev) & (~0x3f)) != (mminor(st1.st_dev) & (~0x3f)))))
-#elif defined(__solaris__)
+#elif defined(__sun__)
 	else if (hwdevs->type == TYPE_SCSI && (st2.st_dev & (~0x7)) != (st1.st_dev & (~0x7)))
 #else
 #  error "Unknown system"
@@ -1314,7 +1331,7 @@ int main(int argc,char **argv)
 	else if (hwdevs->type == TYPE_SCSI)
 #ifdef __linux__	
             config_file_partno = (mminor(st2.st_dev) & 0x0f);
-#elif defined(__solaris__)
+#elif defined(__sun__)
 	    config_file_partno = (st2.st_dev & 7) + 1;
 #else
 #  error "Unknown system"
@@ -1360,7 +1377,7 @@ int main(int argc,char **argv)
     exit(0);
 }
 
-#ifdef __solaris__
+#ifdef __sun__
 
 static errcode_t std_open (const char *name, int flags, io_channel * channel);
 static errcode_t std_close (io_channel channel);
