@@ -15,23 +15,28 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   USA.  */
+
+#ifdef __sun__
+#include "../second/fs/ufs.c"
+#endif
 
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
 #include <stdlib.h>
-#include <features.h>
+#include <unistd.h>
+#include <sys/ioctl.h>
 #ifdef __GLIBC__
 #  include <sys/stat.h>
 #  define _LINUX_STAT_H
+#  define _LINUX_TIME_H
+#  define _LINUX_STRING_H_
 #endif
 #ifdef __linux__
 #  include <linux/fs.h>
 #  include <linux/ext2_fs.h>
-#  ifdef __GLIBC__
-#    define _LINUX_TIME_H
-#  endif
 #  include <scsi/scsi.h>
 #  include <endian.h>
 #  ifdef __GLIBC__
@@ -44,7 +49,7 @@
 #    define mminor(x) (int)((x) & 0xff)
 #    define mmakedev(major, minor) (((major) << 8) | (minor))
 #  endif
-#elif defined (__solaris__)
+#elif defined (__sun__)
 #  include <sys/types.h>
 #  include <sys/stat.h>
 #  include <non-linux/ext2_fs.h>
@@ -167,9 +172,7 @@ void read_sb (char *device, char *bootdev)
     int fd, partno;
     char buff[512];
     struct sun_disklabel *sdl;
-    struct silo_ufs_super_block ufs;
-    struct ext2_super_block sb;	/* Super Block Info */
-    int offset, i;
+    int offset;
 
     if ((fd = open (device, O_RDONLY)) == -1)
 	fatal ("Cannot open superblock on %s", device);
@@ -181,7 +184,7 @@ void read_sb (char *device, char *bootdev)
     doff = 0;
 #ifdef __linux__    
     partno = (int) (*(device + strlen (device) - 1) - '0') - 1;
-#elif defined(__solaris__)
+#elif defined(__sun__)
     partno = (int) (*(device + strlen (device) - 1) - '0');
 #endif
     sdl = (struct sun_disklabel *) &buff;
@@ -196,7 +199,7 @@ void read_sb (char *device, char *bootdev)
 
 int get_partition_blocks (char *device, char *filename)
 {
-    int fd, fd2;
+    int fd;
     int block, i, j, k;
     struct stat st;
     int size;
@@ -223,7 +226,7 @@ int get_partition_blocks (char *device, char *filename)
     }
     blocks[j] = 0;
     nblocks = j;
-#elif defined(__solaris__)
+#elif defined(__sun__)
     ufs_blocks (device, st.st_ino);
 #endif
     close (fd);
@@ -247,7 +250,7 @@ char *find_dev(int number)
     if ((dp = opendir(DEVNAME)) == NULL) return NULL;
     strcpy(name,DEVNAME "/");
     p = strchr (name, 0);
-    while (dir = readdir(dp)) {
+    while ((dir = readdir(dp)) != NULL) {
         strcpy(p,dir->d_name);
         if (stat(name,&s) < 0) return NULL;
         if (S_ISBLK(s.st_mode) && s.st_rdev == number) return name;
@@ -269,13 +272,11 @@ void usage (char *s)
 
 int main(int argc,char **argv)
 {
-    struct stat st1, st2, st3;
-    int fd;
+    struct stat st1;
     int i, j, printblocks = 0;
     char *name;
     char bootdev[1024], bootdev2[1024];
     char *filename;
-    char *p;
 
     name = *argv++;
     argc--;
@@ -309,7 +310,7 @@ int main(int argc,char **argv)
 	if (!p) fatal ("Couldn't find out what device is %s on", filename);
 	strcpy (bootdev, p);
 	strcpy (bootdev2, p);
-#ifdef __solaris__
+#ifdef __sun__
 	if (strlen (p) == strlen ("/dev/dsk/c0t0d0s0")) {
 	    p = strchr (p, 0) - 2;
 	    if (*p == 's' && p[1] >= '0' && p[1] <= '7') {
@@ -352,7 +353,7 @@ int main(int argc,char **argv)
     exit(0);
 }
 
-#ifdef __solaris__
+#ifdef __sun__
 
 static errcode_t std_open (const char *name, int flags, io_channel * channel);
 static errcode_t std_close (io_channel channel);
